'***************************************************************************
'* ATTENTION PLEASE: this code compiles with BASCOM AVR 1.11b and up !     *
'*                              O.N.L.Y.                                   *
'***************************************************************************
'*        YACH - Version 1.1         *
'*************************************
'*    Hexapod with two servos        *
'*        and two feelers            *
'*     "Steers like a cow" ;)        *
'*************************************
'* Portb.1: Middle Pair of Legs      *
'* Portb.0: Front- and Rear Pair     *
'* Portb.2: Left feeler              *
'* Portb.3: Right feeler             *
'*************************************
'*               Init                *
'*************************************

Init:
$crystal = 10000000                  ' Quartz value in Hz

$baud = 9600                         ' Baudrate for the serial port

Ddrb = &B11110011                    ' set portb pin direction for minm & b3
'                                    ' to "in", rest to out

Dim A As Word
Dim H As Byte
Dim B As Byte
Dim Dir As Byte                      ' contains the direction
Dim Newtouch As Byte                 ' =1: move is called by a feeler sub
Dim Feeler As Byte                   ' feeler value, 0=No Contact,
'                                    ' 1= Left, 2= Right, 3= Both
Dim Speed As Integer                 ' speed of movement
Dim Maxf As Byte , Minf As Byte      ' Maxi-/Minimum range (forelegs)
Dim Maxm As Byte , Minm As Byte      ' Maxi-/Minimum range (middlelegs)
Dim Count As Byte                    ' duration of current type of movement

Config Portb = &B11110011            ' configure PORTB as OUTPUT

'*************************************
'*              Main                 *
'*************************************

Main:                                ' Main Routine
    Newtouch = 0                     ' no Contact on Switch-on
    Dir = Rnd(7)                     ' randomly select a direction/action

    Gosub Feelme                     ' check feelers, if "contact" Dir will
'                                    ' be changed accordingly
    Gosub Action                     ' move depending on Dir
Goto Main                            ' keep on walking forever

'*************************************
'*               Action              *
'*************************************

Action:


On Dir Goto Fow , Lef , Rig , Dnc , Bck , Ffw , Pse , F_rgt , F_lft , F_bth

Fow:                                 ' forward
   Maxm = 110
   Minm = 230
   Maxf = 180
   Minf = 150
   Count = 20                        'it will walk forward for a longer time
   Speed = 5
   Gosub Moveme
Return

Lef:                                 'turn left
   Maxm = 145
   Minm = 230
   Maxf = 200
   Minf = 150
   Count = 16
   Speed = 3
   Gosub Moveme
Return

Rig:
   Maxm = 110
   Minm = 145
   Maxf = 165                        ' turn right
   Minf = 120
   Count = 16
   Speed = 3
   Gosub Moveme
Return

Dnc:                                 ' Step Dance :)
   Maxf = 165
   Minf = 165
   Count = 10
   Speed = 7
   Gosub Moveme
Return

Bck:                                 ' backward
   Maxm = 230
   Minm = 110
   Maxf = 180
   Minf = 150
   Count = 10
   Speed = 5
   Gosub Moveme
Return

Ffw:                                 ' fast forward
   Maxm = 110
   Minm = 230
   Maxf = 180
   Minf = 150
   Count = 10
   Speed = 7
   Gosub Moveme
Return

Pse:
    Wait 3
Return

F_non:                               'no contact made - no action done
Return

F_lft:                               ' left feeler touched
   Newtouch = 1                      ' move without testing feelers
   Gosub Dnc                         ' Panic !
   Gosub Bck                         ' shy back
   Gosub Rig
   Newtouch = 0                      ' try the right side
Return

F_rgt:                               ' right feeler touched
   Newtouch = 1                      ' move without testing feelers
   Gosub Dnc                         ' Panic !
   Gosub Bck                         ' shy back
   Gosub Lef
   Newtouch = 0
Return

F_bth:                               ' both feelers touched
   Newtouch = 1                      ' move without testing feelers
   Wait 3                            ' BIG panic :)
   Gosub Dnc
   Gosub Bck                         ' shy back
   Gosub Lef
   Gosub Lef
   Newtouch = 0
Return


'*************************************
'*               Feelme              *
'*************************************

Feelme:
   Feeler = Pinb And &B00001100
   Feeler = Feeler / 4
   If Feeler = 0 Then Return
   Dir = Feeler + 6                  ' => 7,8,9: for the "On .. Goto .."

Return

'*************************************
'*               Moveme              *
'*************************************

Moveme:                              ' approx. values for the range:
'                                    ' 115-179: forward
'                                    ' 187-255: reverse
'                                    ' 180-186: zero position
While Count <> 0
   For H = 0 To 9
      Set Portb.1                    ' start pulse on portb, pin1
      For B = 1 To Maxm
         Waitus 4
      Next B
      Reset Portb.1                  ' end pulse on portb, pin1
      Waitms 10
   Next H

   For A = Minf To Maxf Step Speed   ' The value for "step" will
'                                    ' determine the speed.
'                                    ' counting upward means "forward"
      For H = 0 To 2                 ' give the servo some time to turn
         Set Portb.0                 ' start pulse on portb, pin0
         For B = 1 To A              ' "A" determines the actual position
            Waitus 4                 ' the servo is supposed to go to
         Next B
         Reset Portb.0               ' end pulse on portb, pin0
         Waitms 10                   ' wait a bit ("every 20 msec" ...)
      Next H
      Set Portb.1                    ' start pulse on portb, pin1
      For B = 1 To Maxm
         Waitus 4
      Next B
      Reset Portb.1                  ' end pulse on portb, pin1
      Waitms 10
   Next A

   For H = 0 To 12
      Set Portb.1
      For B = 1 To Minm
         Waitus 4
      Next B
      Reset Portb.1
      Waitms 10
   Next H
   If Newtouch = 0 Then
      Gosub Feelme                   ' quick check: feeler contacted?
      If Dir > 6 Then Return         ' yes?
   End If
   Speed = Speed * -1                ' change direction
   For A = Maxf To Minf Step Speed   ' counting downwards means
'                                    ' moving backwards
'                                    ' all is relative, of course :)
      For H = 0 To 2
         Set Portb.0
         For B = 1 To A
            Waitus 4
         Next B
         Reset Portb.0
         Waitms 10
      Next H

      Set Portb.1
      For B = 1 To Minm
         Waitus 4
      Next B
      Reset Portb.1
      Waitms 10
   Next A
   Count = Count - 1                 ' repeat curr. direction until COUNT=0
   Speed = Speed * -1                ' change direcion again
Wend
Return

' And Yes, I would hate to take part in a dead reckoning contest ;)
End